// ReflectiveFloor_GradientReflection effect
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect generates gradient reflection mapping. The Colour, Colour 1 and Colour 2 properties are used to generate the gradient.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		EyePos;

float4		DiffuseCol;

float4		TextureResolution0;
float4		TextureResolution1;
float4		UVBounds0;
float4 		UVBounds1;

const float cSmoothWaterClip=10;

float4 Prop0
<
string UIName="Floor Colour";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,1,1);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(1,1,1,1);

float4 Prop1
<
string UIName="Transparency Reflection";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.5,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.5,0,0,0);

float4 Prop2
<
string UIName="Transparency Gradient";
float UIMin=0;
float UIMax=0.1;
float3 UIDefault=float3(0.01,0,0);
int UISliders=1;
float UIScale=100;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.01,0,0,0);

float4 Prop3
<
string UIName="Floor Level";
float UIMin=-100;
float UIMax=100;
float3 UIDefault=float3(0,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop4
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop5
<
string UIName="Colour 1";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.3,0.3,0.3);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(0.3,0.3,0.3,1);

float4 Prop6
<
string UIName="Colour 2";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.1,0.1,0.1);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(0.1,0.1,0.1,1);

float4 Prop7
<
string UIName="Gradient / Smooth";
float UIMin=0;
float UIMax=10;
float3 UIDefault=float3(8,1,1);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(8,1,1,1);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float3 tex:		TEXCOORD0;
	float4 worldpos:	TEXCOORD1;
	float4 worldposR:	TEXCOORD2;
	float3 normal:		TEXCOORD3;
	float2 tex2:		TEXCOORD4;	
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop4.x==0 && Prop4.y==0 && Prop4.z==0) return a;

	float sa=sin(Prop4.x);
	float ca=cos(Prop4.x);
	float sb=sin(Prop4.y);
	float cb=cos(Prop4.y);
	float sg=sin(Prop4.z);
	float cg=cos(Prop4.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

//spiegelbeeld
VS_OUTPUT VS1(VS_INPUT input){
	VS_OUTPUT output;  
	
	float4 pos  = mul(input.position, World);

	float4 posR = pos;
	posR.y = 2*Prop3.x - pos.y;		
	
	output.worldpos 	= pos;
	output.worldposR 	= posR;
	output.position 	= mul(posR, ViewProjection);

	float3 NormalVector	= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector	= normalize(EyePos - pos);

	output.tex		= RotateReflection(reflect(NormalVector,EyeVector));
	output.tex2		= input.tex;

	output.normal		= NormalVector;
	return output;
}

//normaal beeld
VS_OUTPUT VS2(VS_INPUT input){
	VS_OUTPUT output;      

	float4 pos = mul(input.position, World);

	output.worldpos 	= pos;
	output.worldposR 	= pos;
	output.position 	= mul(pos, ViewProjection);

	float3 NormalVector	= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector	= normalize(EyePos - pos);

	output.tex		= RotateReflection(reflect(NormalVector,EyeVector));
	output.tex2		= input.tex;

	output.normal		= NormalVector;
	return output;
}

float4 DoIt(VS_OUTPUT input, bool inMirror, float2 inTex){

	float f=input.tex.y;
	if(f<-1) f=-1;
	if(f>1) f=1;

	f=0.5*(f+1);

	float b=Prop7.y * 0.001;

	float4 r=float4(1,1,1,1);

	if(f<=(0.5-b)){
		r=DiffuseCol;
	}

	if(f>=(0.5-b) && f<=(0.5+b)){
		float f2=(f-(0.5-b))/(2*b);
		r=DiffuseCol+f2*(Prop5-DiffuseCol);
	}

	if(f>=(0.5+b)){
		f=(f-(0.5+b))/(0.5-b);
		f=f*Prop7.x;
		if(f>1) f=1;
		r=Prop5+f*(Prop6-Prop5);
	}

	if(TextureResolution0.x>0){
		r*=tex2D(MyTextureSampler0,inTex);
	}

	return r;
}


//spiegelbeeld
PS_OUTPUT PS1(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=DoIt(input,true,input.tex2);

	if(input.worldposR.y > Prop3.x){
		c.w*=(1-(input.worldposR.y-Prop3.x)*cSmoothWaterClip);
		if(c.w<0) c.w=0;
	}

	c.w*=(1-Prop1.x);

	float f=1-(-(input.worldposR.y-Prop3.x) * Prop2.x);
	if(f<0) f=0;
	if(f>1) f=1;
	c.w*=f;
	c.w*=DiffuseCol.w;

	c.xyz *= Prop0.xyz;

	output.color=c;
	return output;
}

//normaal beeld
PS_OUTPUT PS2(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=DoIt(input,false,input.tex2);

	if(input.worldpos.y < Prop3.x){
		c.w*=(1-(Prop3.x-input.worldposR.y)*cSmoothWaterClip);
		if(c.w<0) c.w=0;
	}

	c.w*=DiffuseCol.w;
	
	output.color = c;
	return output;
}

technique Shader
{
    //mirror: alleen z-buffer schrijven
    pass P1
    {
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;
	ALPHAARG1[0]	= DIFFUSE;
	ALPHAOP[0]	= SELECTARG1;
	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;
	CullMode	= CW;
	ZFUNC		= LESSEQUAL;

	ColorWriteEnable=0;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //mirror: RGB
    pass P0
    {
	ColorWriteEnable=7;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //normaal    
    pass P1
    {
	CullMode	= CCW;
        VertexShader	= compile vs_3_0 VS2();
        PixelShader	= compile ps_3_0 PS2();
    }
}

technique AlphaShader
{
    //mirror: alleen z-buffer schrijven
    pass P1
    {
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;
	ALPHAARG1[0]	= DIFFUSE;
	ALPHAOP[0]	= SELECTARG1;
	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;
	CullMode	= CW;
	ZFUNC		= LESSEQUAL;

	ColorWriteEnable=0;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //mirror: RGB
    pass P0
    {
	ColorWriteEnable=8;
		
        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //normaal    
    pass P1
    {
	CullMode	= CCW;
        VertexShader	= compile vs_3_0 VS2();
        PixelShader	= compile ps_3_0 PS2();
    }
}

