// ColourMap_Cubemap
//
// Copyright Outerspace Software, all rights reserved

string description = "This effect combines colourmapping with cubemapping. The first texture must be a colourmap, the second a cubemap.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4 Prop0
<
string UIName="Lighting Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.9,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.9,0,0,0);

float4 Prop1
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state {
	Texture = <MyTexture0>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

texture MyTexture1;
samplerCUBE MyTextureSampler1 = sampler_state {
	Texture = <MyTexture1>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float4 diffuse:		COLOR;
	float2 tex0:		TEXCOORD0;
	float3 tex1:		TEXCOORD1;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop1.x==0 && Prop1.y==0 && Prop1.z==0) return a;

	float sa=sin(Prop1.x);
	float ca=cos(Prop1.x);
	float sb=sin(Prop1.y);
	float cb=cos(Prop1.y);
	float sg=sin(Prop1.z);
	float cg=cos(Prop1.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;
	float4 Pos=mul(input.position, World);
	output.position = mul(Pos, ViewProjection);

	float3 NormalVector		= normalize(mul(input.normal,WorldInverseTranspose));
	float3 LightVector		= normalize(LightPos - Pos);
	float3 EyeVector		= normalize(EyePos   - Pos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);
	
	//ambient component
	output.diffuse = AmbientCol;

	//diffuse component
	output.diffuse += DiffuseCol * saturate(dot(NormalVector, LightVector));
	
	//specular component	
	output.diffuse += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);
	
	output.diffuse.xyz *= Prop0.x;
	output.diffuse.w = DiffuseCol.w;

	output.tex0 = input.tex;
	output.tex1 = RotateReflection(reflect(NormalVector,EyeVector));
	return output;
	
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;
	
	float4 A=input.diffuse*tex2D(MyTextureSampler0, input.tex0);
	float4 B=DiffuseCol*texCUBE(MyTextureSampler1,normalize(input.tex1));
	 
	output.color.xyz = A*Prop0.x + B*(1-Prop0.x);

	output.color.w = A.w;

	return output;
}

technique Shader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}
