// EternalZoom.fx
//
// Copyright Outerspace Software, all rights reserved

string description = "Continuous Zoom Animation";

bool optimization=false;

float4x4	WorldViewProjection;
float 		Time;
float4		DiffuseCol;

float4 Prop0
<
string UIName="Resolution";
float UIMin=1;
float UIMax=10;
float3 UIDefault=float3(6,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=true;
int UIWidget=0;
> = float4(6,0,0,0);

float4 Prop1
<
string UIName="Speed";
float UIMin=0;
float UIMax=10;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float4 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

VS_OUTPUT VS0(VS_INPUT input){
	VS_OUTPUT output;

	int Period=Prop0.x;
	if(Period<=0) Period=6;
	float P=2;
	float Res=pow(P,Period);

	float Speed=Prop1.x;
	if(Speed<=0) Speed=1;

	float t = fmod(Time*Speed,Period);

	float2 uv = input.tex;

	uv -= float2(0.5,0.5);
	uv /= pow(P,t);
	uv += float2(0.5,0.5);

	output.position = mul(input.position,WorldViewProjection);
	output.diffuse	= float4(1,1,1,DiffuseCol.w);
	output.tex 	= uv;
	return output;
}

VS_OUTPUT VS1(VS_INPUT input){
	VS_OUTPUT output;

	int Period=Prop0.x;
	if(Period<=0) Period=6;
	float P=2;
	float Res=pow(P,Period);

	float Speed=Prop1.x;
	if(Speed<=0) Speed=1;

	float t = fmod(Time*Speed,Period);

	float2 uv = input.tex;

	uv -= float2(0.5,0.5);
	uv /= pow(P,t);
	uv += float2(0.5,0.5);

	uv += float2(0.5,0.5) / Res;
	uv *= Res;

	output.position = mul(input.position,WorldViewProjection);

	float a = (DiffuseCol.w*t)/Period;

	output.diffuse	= float4(1,1,1,a);
	output.tex 	= uv;
	return output;
}

technique pixels{

	pass p0{
		ADDRESSU[0] = WRAP;
		ADDRESSV[0] = WRAP;    

		VertexShader = compile vs_1_1 VS0();
	} 

	pass p1{
		Alphablendenable=	TRUE;
		SRCBLEND	=	SRCALPHA;
		DESTBLEND	=	INVSRCALPHA;

		ADDRESSU[0]	=	WRAP;
		ADDRESSV[0]	=	WRAP;    

		ALPHAARG1[0]	=	TEXTURE;
		ALPHAARG2[0]	=	DIFFUSE;
		ALPHAOP[0]	=	MODULATE;

		ZFunc		=	ALWAYS;

	        VertexShader = compile vs_2_0 VS1();
	}

}
